include IntelliJ::Helpers
require 'json'

module IntelliJ
  module Install
    def summarize_release(params)
      source = params.source
      checksum = params.checksum
      version = params.version

      unless params.source
        release = get_release_data(params.version, params.edition)
        version = release['version']
        source, checksum = get_release_download_info(release)
      end

      [source, checksum, version]
    end

    def download_package(source, checksum, user, group)
      Chef::Log.info("Downloading IntelliJ from #{source} (checksum: #{checksum})")
      package_path = Chef::Platform.windows? ? "#{Chef::Config[:file_cache_path]}/#{::File.basename(source)}" : "/tmp/#{::File.basename(source)}"

      remote_file 'Download IntelliJ package' do
        source source
        checksum checksum
        path package_path
        user user
        group group
      end

      package_path
    end

    def install_package(path, package_path, user, group)
      directory 'Make sure the installation path for the IntelliJ package exists' do
        path path
        recursive true
        user user
        group group
      end

      zipfile package_path do
        into path
        only_if { ::File.extname(package_path) == '.zip' }
      end

      tar_extract package_path do
        action :extract_local
        only_if { package_path.end_with?('.tar.gz') }
        target_dir path
        creates "#{path}/bin"
        tar_flags [ '--strip-components 1' ]
        user user
        group group
      end
    end

    def remove_package(path)
      directory 'Remove existing IntelliJ installation' do
        path path
        recursive true
        action :delete
        only_if { ::File.exist?(path) }
      end
    end

    def intellij_installed?(path, version, edition)
      local_version, local_edition = get_local_intellij_info(path)
      edition_code = node['intellij']['edition_code'][edition]

      if local_version
        if version == local_version && edition_code == local_edition
          Chef::Log.info("The IntelliJ #{edition} edition in #{path} is already installed in the version #{version}")
        else
          Chef::Log.warn("The IntelliJ in #{path} is already installed, but does not match requirements (edition: #{edition_code} => #{local_edition}, version: #{version} => #{local_version})")
        end
      end

      ::File.exist?("#{path}/build.txt")
    end

    def get_local_intellij_info(path)
      result = nil
      if ::File.exist?("#{path}/product-info.json")
        info = JSON.parse(::File.read("#{path}/product-info.json"))
        result = [info['version'], info['productCode']]
      end
      result
    end
  end
end
