module IntelliJ
  module Workspace
    def prepare_workspace(path, user, group)
      directory 'Make sure the workspace directory exists' do
        path path
        recursive true
        user user
        group group
      end

      directory 'Make sure the config directory exists' do
        path format(node['intellij']['workspace']['config_dir'], path: path)
        recursive true
        user user
        group group
      end

      variables = {
        CONFIG_PATH: format(node['intellij']['workspace']['config_dir'], path: path),
      }

      template 'Create idea.properties for the new workspace' do
        source 'workspace/idea.properties.erb'
        path format(node['intellij']['workspace']['config']['properties'], path: path)
        variables variables
        cookbook 'intellij'
        user user
        group group
      end
    end

    def setup_default_project(project_path, path, user, group)
      file = format(node['intellij']['workspace']['config']['recent-projects'], path: path)

      directory 'Make sure the parent directory for default project options exists' do
        path File.dirname(file)
        recursive true
        user user
        group group
      end

      template 'Create default file in case it does not exist' do
        source 'workspace/recentProjects.xml'
        path file
        not_if { ::File.exist?(file) }
        cookbook 'intellij'
        user user
        group group
      end

      xml_edit file do
        parent '/application/component[@name="RecentProjectsManager"]'
        target '/application/component[@name="RecentProjectsManager"]/option[@name="openPaths"]'
        fragment <<-EOH
          <option name="openPaths">
            <list>
              <option value="#{project_path}" />
            </list>
          </option>
        EOH
        action :append_if_missing
      end

      xml_edit file do
        parent '/application/component[@name="RecentProjectsManager"]'
        target '/application/component[@name="RecentProjectsManager"]/option[@name="lastPath"]'
        fragment "<option name=\"lastPath\" value=\"#{project_path}\" />"
        action :append_if_missing
      end
    end

    def setup_color_scheme(color_scheme, path, user, group)
      color_file = format(node['intellij']['workspace']['config']['color-scheme'], path: path)
      laf_file = format(node['intellij']['workspace']['config']['laf'], path: path)

      directory 'Make sure the parent directory for the color scheme options exists' do
        path File.dirname(color_file)
        recursive true
        user user
        group group
      end

      template 'Create default colors scheme file in case it does not exist' do
        source 'workspace/colors.scheme.xml'
        path color_file
        not_if { ::File.exist?(color_file) }
        cookbook 'intellij'
        user user
        group group
      end

      template 'Create default LAF file in case it does not exist' do
        source 'workspace/laf.xml'
        path laf_file
        not_if { ::File.exist?(laf_file) }
        cookbook 'intellij'
        user user
        group group
      end

      xml_edit color_file do
        parent '/application/component[@name="EditorColorsManagerImpl"]'
        target '/application/component[@name="EditorColorsManagerImpl"]/global_color_scheme'
        fragment "<global_color_scheme name=\"#{color_scheme}\" />"
        action :append_if_missing
      end

      xml_edit laf_file do
        parent '/application/component[@name="LafManager"]'
        target '/application/component[@name="LafManager"]/laf'
        fragment node['intellij']['workspace']['laf'][color_scheme]
        action :append_if_missing
      end
    end

    def create_shortcut(path, ij_path, user, group)
      ruby_block 'Prepare creating the workspace' do
        block do
          properties_path = format(node['intellij']['workspace']['config']['properties'], path: path)

          generate_shortcut(ij_path, path, properties_path, user, group)
        end
      end
    end

    def generate_shortcut(ij_path, path, properties, user, group)
      windows_shortcut format(node['intellij']['helpers']['shortcut_name'], path: path) do
        cwd format(node['intellij']['helpers']['shortcut_cwd'], path: path)
        target 'C:\Windows\System32\cmd.exe'
        arguments format(node['intellij']['helpers']['shortcut_arguments'], path: properties, ij_path: ij_path)
        iconlocation format(node['intellij']['helpers']['icon_location'], path: ij_path)
        only_if { Chef::Platform.windows? }
      end

      variables = {
        PATH: properties,
        INTELLIJ_PATH: ij_path,
      }

      template "#{path}/idea.sh" do
        source 'workspace/start-script.sh.erb'
        user user
        group group
        mode '0755'
        variables variables
        not_if { Chef::Platform.windows? }
      end
    end
  end
end
